/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "cons_buf.h"
#include "pctextmode.h"


#include "vgafont16.h"

const uint32_t abydos_cga_palette16[16] = {
    0x00000000,    /* black */
    0x000000aa,    /* dark blue */
    0x0000aa00,    /* dark green */
    0x0000aaaa,    /* dark cyan */
    0x00aa0000,    /* dark red */
    0x00aa00aa,    /* dark magenta */
    0x00aa5500,    /* brown */
    0x00aaaaaa,    /* light gray */
    0x00555555,    /* dark gray */
    0x005555ff,    /* light blue */
    0x0055ff55,    /* light green */
    0x0055ffff,    /* light cyan */
    0x00ff5555,    /* light red */
    0x00ff55ff,    /* light magenta */
    0x00ffff55,    /* yellow */
    0x00ffffff     /* white */
};


cairo_bool_t
abydos_console_has_blink(Console *cons){
    int size = cons_buf_width(cons) * cons_buf_height(cons);
    const ConsoleCell *p = cons_buf_buffer(cons);
    while(size>0){
        if(p->attr & 0x80)
            return 1;
        --size;
        ++p;
    }
    return 0;
}


static void
_draw_character(uint32_t *dst,int rowskip,int ch,uint32_t bg,uint32_t fg,int letter_spacing)
{
    uint8_t *fontchar = &vga_rom_16[ch*16];
    int ninth = letter_spacing>8 && ch>=192 && ch<224;
    int x;
    int y;
    for(y=0;y<16;++y){
        int pixels = *fontchar++;
        for(x=0;x<8;++x){
            pixels <<= 1;
            *dst++ = pixels&0x100 ? fg : bg;
        }
        *dst++ = ninth && pixels&0x100 ? fg : bg;
        dst += rowskip;
    }
}


cairo_surface_t *
abydos_image_surface_from_console(Console *cons, AbydosConsoleState state, int letter_spacing)
{
    const ConsoleCell *src = cons_buf_buffer(cons);
    int width = cons_buf_width(cons);
    int height = cons_buf_height(cons);
    cairo_surface_t *surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24,width*letter_spacing,height*16);
    uint32_t *dst = (uint32_t*)cairo_image_surface_get_data(surface);
    int stride = cairo_image_surface_get_stride(surface)/sizeof(uint32_t);
    int rowskip = stride-letter_spacing;
    int x;
    int y;
    for(y=0;y<height;++y){
        for(x=0;x<width;++x){
            ConsoleCell cell = *src++;
            uint32_t bg;
            uint32_t fg;
            if(S_ICE==state){
                bg = abydos_cga_palette16[(cell.attr>>4)&15];
            }else{
                bg = abydos_cga_palette16[(cell.attr>>4)&7];
            }
            if(!(S_OFF==state && (cell.attr&0x80))){
                fg = abydos_cga_palette16[(cell.attr)&15];
                _draw_character(dst+x*letter_spacing,rowskip,cell.ch,bg,fg,letter_spacing);
            }else{
                _draw_character(dst+x*letter_spacing,rowskip,' ',bg,bg,letter_spacing);
            }
        }
        dst += stride*16;
    }
    cairo_surface_mark_dirty(surface);
    return surface;
}
