/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <string.h>                     /* memcpy() memmove() */

#include "cycle.h"


int
color_cycle_total_frames(color_cycle_t *cc, int count)
{
    return cc->last - cc->first + 1;
}


double
color_cycle_total_duration(color_cycle_t *cc, int count)
{
    return cc->duration * (cc->last - cc->first + 1);
}


void
color_cycle_apply_step(color_cycle_t *cc, void *palette, int colorsize)
{
    char color[16];
    if(CCD_LEFT == cc->direction){
        memcpy(color, palette + cc->first*colorsize, colorsize);
        memmove(palette + cc->first*colorsize, palette + (cc->first+1)*colorsize, (cc->last - cc->first)*colorsize);
        memcpy(palette + cc->last*colorsize, color, colorsize);
    }else{
        memcpy(color, palette + cc->last*colorsize, colorsize);
        memmove(palette + (cc->first+1)*colorsize, palette + cc->first*colorsize, (cc->last - cc->first)*colorsize);
        memcpy(palette + cc->first*colorsize, color, colorsize);
    }
}


void color_cycle_apply_time(color_cycle_t *cc, double time, void *palette, int colorsize)
{
    while(time >= cc->duration){
        color_cycle_apply_step(cc, palette, colorsize);
        time -= cc->duration;
    }
}
