/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <string.h>                     /* memset(), memcpy() */

#include "bitplane.h"


static void
_copy_bitplanes(cairo_surface_t *surface, int bpp,
                uint8_t **bitplane,
                int width, int height, int stride, int depth,
                uint8_t *palette, int palette_stepping)
{
    uint8_t *dst = cairo_image_surface_get_data(surface);
    int rowskip = cairo_image_surface_get_stride(surface) - width*bpp;
    int offs = 0;
    while(height-->0){
        int x;
        for(x=0;x<stride;++x){
            uint8_t b[8];
            int plane;
            int i;
            memset(b,0,8);
            for(plane=0;plane<depth;++plane)
                for(i=0;i<8;++i)
                    if(bitplane[plane][offs+x] & (128>>i))
                        b[i] |= 1<<plane;
            for(i=0;i<8;++i){
                if(x*8+i >= width)
                    break;
                memcpy(dst, palette + b[i]*bpp, bpp);
                dst += bpp;
            }
        }
        offs += stride;
        dst += rowskip;
        palette += palette_stepping*bpp;
    }
    cairo_surface_mark_dirty(surface);
}


cairo_surface_t *
abydos_image_surface16_from_bitplanes(uint8_t **bitplane,
                                      int width, int height, int stride, int depth,
                                      uint16_t *palette, int palette_stepping)
{
    cairo_surface_t *surface = cairo_image_surface_create(CAIRO_FORMAT_RGB16_565, width, height);
    _copy_bitplanes(surface, 2, bitplane, width, height, stride, depth, (uint8_t*)palette, palette_stepping);
    return surface;
}


cairo_surface_t *
abydos_image_surface32_from_bitplanes(uint8_t **bitplane,
                                      int width, int height, int stride, int depth,
                                      uint32_t *palette, int palette_stepping)
{
    cairo_surface_t *surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, width, height);
    _copy_bitplanes(surface, 4, bitplane, width, height, stride, depth, (uint8_t*)palette, palette_stepping);
    return surface;
}
